// Copyright (c) 2000
//      Maurice Castro (maurice@serc.rmit.edu.au)
//      All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 
// This software is provided by the original author and contributors ``as
// is'' and any express or implied warranties, including, but not limited
// to, the implied warranties of merchantability and fitness for a
// particular purpose are disclaimed.  In no event shall the authors or
// contributors be liable for any direct, indirect, incidental, special,
// exemplary, or consequential damages (including, but not limited to,
// procurement of substitute goods or services; loss of use, data, or
// profits; or business interruption) however caused and on any theory of
// liability, whether in contract, strict liability, or tort (including
// negligence or otherwise) arising in any way out of the use of this
// software, even if advised of the possibility of such damage.
//
// $Id: GetPage.java,v 1.2 2000/11/12 23:02:46 maurice Exp $

import java.net.*;
import java.awt.*;
import java.io.*;
import java.lang.*;
import java.util.*;

public class GetPage
{
	private TextArea Status;
	private int level;
	private URL Document;
	private String destdir;
	private XlateURL xlate;
	private Vector fetched;
	private static PageXlate px;

	private void msg(String S)
	{
		Status.appendText(S + "\n" );
	}

	public GetPage(TextArea status, String url, int level, String destdir, int mode)
	{
		this.Status = status;
		this.destdir = destdir;
		this.xlate = new XlateURL(mode);
		this.fetched = new Vector();
		this.px = new PageXlate();
		init(url, level);
	}

	private GetPage(TextArea status, String url, int level, String destdir, XlateURL xlate, Vector fetched)
	{
		this.Status = status;
		this.xlate = xlate;
		this.destdir = destdir;
		this.fetched = fetched;
		init(url, level);
	}

	public String toString()
	{
		return(Document + " " + level + " " + destdir + " " + xlate);
	}

	private void init(String URL, int level)
	{
		this.level = level;
		try 
		{
			Document = new URL(URL);
		}
		catch (Exception e) 
		{
			msg(e.toString());
		}
	}

	private boolean AlreadyFetched(String f)
	{
		return (fetched.contains(f));
	}
	
	private void Fetched(String f)
	{
		fetched.addElement(f);
	}

	public void Fetch()
	{
		String OutFile;
		if (Document == null)
			return;
		OutFile = xlate.lookup(Document, true, false);
		if (AlreadyFetched(OutFile))
			return;
		msg("Fetch: " + Document.toString());
		try
		{
			URLConnection Connect = Document.openConnection();
			Connect.connect();
			InputStream Content = Connect.getInputStream();
			File out = new File(destdir, OutFile);
			FileOutputStream Fout = new FileOutputStream(out);
			if (Connect.getContentType().equals("text/html"))
			{
				msg("Process: Rewriting: " + Document.toString() + " --> " + out);
				xlate.setLevel(level);
				Vector v = new Vector();
				xlate.setVector(v);
				px.rewrite(Content, Fout, Connect.getURL(), xlate);
				Enumeration urls = v.elements();
				while (urls.hasMoreElements())
				{
					URL url = (URL) urls.nextElement();
					GetPage Page = new GetPage(Status, url.toString(), level-1, destdir, xlate, fetched);
					Page.Fetch();
				}
			}
			else
			{
				msg("Process: Copying: " + Document.toString() + " --> " + out);
				px.copy(Content, Fout);
			}
		}
		catch (Exception e) 
		{
			msg("Error: " + e.toString());
		}
		Fetched(OutFile);
	}
}
