// Copyright (c) 2000
//      Maurice Castro (maurice@serc.rmit.edu.au)
//      All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 
// This software is provided by the original author and contributors ``as
// is'' and any express or implied warranties, including, but not limited
// to, the implied warranties of merchantability and fitness for a
// particular purpose are disclaimed.  In no event shall the authors or
// contributors be liable for any direct, indirect, incidental, special,
// exemplary, or consequential damages (including, but not limited to,
// procurement of substitute goods or services; loss of use, data, or
// profits; or business interruption) however caused and on any theory of
// liability, whether in contract, strict liability, or tort (including
// negligence or otherwise) arising in any way out of the use of this
// software, even if advised of the possibility of such damage.
//
// $Id: MainWindow.java,v 1.5 2000/11/13 03:52:41 maurice Exp $

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class MainWindow extends Frame
{
	TextArea status;
	TextField baseurl;
	TextField depth;
	TextField destination;
	Choice opt;
	PageFetcher pf;
	int STATUSROWS = 10, COLS = 30;

	private Panel Makelabeledpanel( String S, Component C )
	{
		Panel p = new Panel();
		p.setLayout( new BorderLayout () );
		Label l = new Label(S);
		Panel p1 = new Panel();
		p1.add(l);
		Panel p2 = new Panel();
		p2.add(C);
		p.add( "North", p1);
		p.add( "Center", p2);
		return(p);
	}

	public MainWindow ()
	{
		super("Snaffle");
		pf = new PageFetcher();
		this.setLayout(new FlowLayout());
		Panel ctrls = new Panel();
		ctrls.setLayout(new BorderLayout());
		Panel p = new Panel();
		p.setLayout(new BorderLayout());
		Panel p2 = new Panel();
		baseurl = new TextField( COLS );
		p.add("North", Makelabeledpanel("Base URL", baseurl));
		depth = new TextField( 2 );
		p2.add(Makelabeledpanel("Depth", depth) );
		opt = new Choice();
		// order is important as we use the indices in XlateURL
		opt.add("Strict Tree");
		opt.add("Named Host Only");
		opt.add("Any");
		p2.add("South", Makelabeledpanel("Recover", opt) );
		p.add("South", p2);
		ctrls.add( "North", p);
		destination = new TextField ( COLS );
		ctrls.add( "Center", Makelabeledpanel("Destination", destination));
		Button fetch = new Button("Fetch");
		fetch.setActionCommand("Fetch");
		Button stop = new Button("Stop");
		stop.setActionCommand("Stop");
		p = new Panel();
		p.add(fetch);
		p.add(stop);
		ctrls.add( "South", p);
		this.add( ctrls );
		status = new TextArea ( STATUSROWS, COLS );
		this.add( Makelabeledpanel("Status", status));
		status.setEditable(false);
		Menu m = new Menu("File");
		m.add(new MenuItem("Load"));
		m.add(new MenuItem("Save"));
		m.add(new MenuItem("Exec"));
		m.addSeparator();
		m.add(new MenuItem("Close", new MenuShortcut(KeyEvent.VK_E)));
		Menu pr = new Menu("Proxy");
		pr.add(new MenuItem("Set Proxy"));
		Menu h = new Menu("Help");
		h.add(new MenuItem("About"));
		MenuBar mb = new MenuBar();
		mb.add(m);
		mb.add(pr);
		mb.setHelpMenu(h);
		setMenuBar(mb);
		this.pack();
		this.show();
		// if it occupies over 2/3 of a screen dimension make it all
		Dimension s = getToolkit().getScreenSize();
		Dimension w = getSize();
		Dimension t = new Dimension(w);
		if (s.height <= 3*w.height/2) 
			t.height = s.height;
		if (s.width <= 3*w.width/2) 
			t.width = s.width;
		setSize(t);
	}

	public boolean handleEvent ( Event event)
	{
		switch (event.id) 
		{
			case Event.WINDOW_DESTROY:
				dispose();
				System.exit(0);
				return(true);
			case Event.ACTION_EVENT:
				if (event.arg.equals("Close"))
				{
					dispose();
	                                System.exit(0);
                                	return(true);
				}
				else if (event.arg.equals("Stop"))
				{
					pf.end();
					return(true);
				}
				else if (event.arg.equals("About"))
				{
					About a = new About(this);
					a.show();
					return(true);
				}
				else if (event.arg.equals("Set Proxy"))
				{
					Proxy a = new Proxy(this);
					a.show();
					return(true);
				}
				else if (event.arg.equals("Load"))
				{
					FileDialog fd = new FileDialog(this, "Load Settings", FileDialog.LOAD);
					fd.show();
					if (fd.getFile() == null)
						return(true);
					try
					{
						File file = new File(fd.getDirectory(), fd.getFile());
						BufferedReader r = new BufferedReader(new FileReader(file));
						baseurl.setText(r.readLine());
						destination.setText(r.readLine());
						depth.setText(r.readLine());
						int option = 0;
						try
						{
							option = (new Integer(r.readLine())).intValue();
						}
						catch ( Exception e ) { }
						opt.select(option);
						r.close();
					}
					catch (Exception e)
					{
						status.setText("Unable to load file\n" + e);
					}
					return(true);
				}
				else if (event.arg.equals("Exec"))
				{
					FileDialog fd = new FileDialog(this, "Execute File", FileDialog.LOAD);
					fd.show();
					Vector v = new Vector();
					if (fd.getFile() == null)
						return(true);
					try
					{
						File file = new File(fd.getDirectory(), fd.getFile());
						BufferedReader r = new BufferedReader(new FileReader(file));
						while (r.ready())
						{
							String BaseURL = r.readLine();
							String dest = r.readLine();
							int level = 0;
							try
							{
								level = (new Integer(r.readLine())).intValue();
							}
							catch (Exception e) {}
							int option = 0;
							try
							{
								option = (new Integer(r.readLine())).intValue();
							}
							catch (Exception e) {}
							opt.select(option);
							GetPage page = new GetPage(status, BaseURL, level, dest, option);
							v.addElement(page);
						}
						r.close();
						pf.begin(status, v);
					}
					catch (Exception e)
					{
						status.setText("Unable to execute file\n" + e);
					}
					return(true);
				}
				else if (event.arg.equals("Save"))
				{
					FileDialog fd = new FileDialog(this, "Save Settings", FileDialog.SAVE);
					fd.show();
					if (fd.getFile() == null)
						return(true);
					try
					{
						File file = new File(fd.getDirectory(), fd.getFile());
						BufferedWriter w = new BufferedWriter(new FileWriter(file));
						String s = baseurl.getText();
						w.write(s, 0, s.length());
						w.newLine();
						s = destination.getText();
						w.write(s, 0, s.length());
						w.newLine();
						s = depth.getText();
						w.write(s, 0, s.length());
						w.newLine();
						s = Integer.toString(opt.getSelectedIndex());
						w.write(s, 0, s.length());
						w.newLine();
						w.close();
					}
					catch (Exception e)
					{
						status.setText("Unable to save to file\n" + e);
					}
					return(true);
				}
				else if (event.arg.equals("Fetch"))
				{
					String BaseURL = baseurl.getText();
					int level = 0;
					try
					{
						level = Integer.valueOf(depth.getText()).intValue();
					}
					catch ( Exception e ) { }
					String dest = destination.getText();
					int FileRef = 1;
					if (!(new File(dest)).isDirectory())
					{
						status.setText("Not a Directory " + dest);
						return(true);
					}
					GetPage page = new GetPage(status, BaseURL, level, dest, opt.getSelectedIndex());
					Vector v = new Vector();
					v.addElement(page);
					pf.begin(status, v);
					return(true);
				}
		}
		return(false);
	}
}

