// Copyright (c) 2000
//      Maurice Castro (maurice@serc.rmit.edu.au)
//      All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 
// This software is provided by the original author and contributors ``as
// is'' and any express or implied warranties, including, but not limited
// to, the implied warranties of merchantability and fitness for a
// particular purpose are disclaimed.  In no event shall the authors or
// contributors be liable for any direct, indirect, incidental, special,
// exemplary, or consequential damages (including, but not limited to,
// procurement of substitute goods or services; loss of use, data, or
// profits; or business interruption) however caused and on any theory of
// liability, whether in contract, strict liability, or tort (including
// negligence or otherwise) arising in any way out of the use of this
// software, even if advised of the possibility of such damage.
//
// $Id: PageXlate.java,v 1.4 2000/12/11 04:32:34 maurice Exp $

import java.net.*;
import java.awt.*;
import java.io.*;
import java.lang.*;
import java.util.*;

public class PageXlate
{
	public void copy(InputStream Fin, OutputStream Fout) throws java.io.IOException
	{
		byte b[] = new byte[1];
		while (Fin.read(b) != -1)
		{
			Fout.write(b);
		}
		Fout.close();
		Fin.close();
	}

	public void rewrite(InputStream Fin, OutputStream Fout, URL Context, XlateURL xlate) throws java.io.IOException
	{
		int state = 0;
		StringBuffer hold = new StringBuffer();
		byte b[] = new byte[1];
		char c;

		while (Fin.read(b) != -1)
		{
			c = (char) b[0];
			switch (state)
			{
				case 0:
					if (c == '<')
					{
						state = 1;
						hold.append(c);
					}
					else
					{
						Fout.write(b);
					}
					break;
				case 1: 
					// <
					if (c == '!')
					{
						state = 200;
						hold.append(c);
					}
					else if (c == 'A' || c == 'a')
					{
						state = 300;
						hold.append(c);
					}
					else if (c == 'I' || c == 'i')
					{
						state = 400;
						hold.append(c);
					}
					else if (c == 'F' || c == 'f')
					{
						state = 500;
						hold.append(c);
					}
					else if (c == 'B' || c == 'b')
					{
						state = 600;
						hold.append(c);
					}
					else 
					{
						// transition to 0
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 200:
					// <!
					if (c == '-')
					{
						state = 201;
						hold.append(c);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 201:
					// <!-
					if (c == '-')
					{
						state = 202;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 202:
					// <!--
					if (c == '-')
					{
						state = 203;
						hold.append(c);
					}
					else 
					{
						// in comment copyout
						Fout.write(b);
					}
					break;
				case 203:
					// <!--.*-
					if (c == '-')
					{
						state = 204;
						hold.append(c);
					}
					else 
					{
						state = 202;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 204:
					// <!--.*--
					if (c == '>')
					{
						// end comment
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == '-')
					{
						hold.append(c);
					}
					else 
					{
						state = 202;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 300:
					// <[Aa]
					if (Character.isWhitespace(c))
					{
						state = 301;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 301:
					// <[Aa]\w+
					// starting on white
					if (c == '>')
					{
						state = 0;
						Fout.write(b);
					}
					else if (c == 'h' || c == 'H')
					{
						state = 303;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						Fout.write(b);
					}
					else 
					{
						// in a record but lead char
						// not whitespace or [hH]
						state = 302;
						Fout.write(b);
					}
					break;
				case 302:
					// <[Aa]\w+[^Hh]+
					// gobbel in a record
					if (Character.isWhitespace(c))
					{
						state = 301;
						Fout.write(b);
					}
					else if (c == '>')
					{
						state = 0;
						Fout.write(b);
					}
					else
					{
						Fout.write(b);
					}
					break;
				case 303:
					// <\w*[Aa]\w[Hh]
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == 'r' || c == 'R')
					{
						state = 304;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						state = 301;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 302;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 304:
					// <\w*[Aa]\w[Hh][Rr]
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == 'e' || c == 'E')
					{
						state = 305;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						state = 301;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 302;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 305:
					// <\w*[Aa]\w[Hh][Rr][Ee]
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == 'f' || c == 'F')
					{
						state = 306;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						state = 301;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 302;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 306:
					// <\w*[Aa]\w[Hh][Rr][Ee][Ff]\w*
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == '=')
					{
						state = 307;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (Character.isWhitespace(c))
					{
						hold.append(c);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 307:
					// <\w*[Aa]\w[Hh][Rr][Ee][Ff]\w*=\w*
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (Character.isWhitespace(c))
					{
						Fout.write(b);
					}
					else if (c == '"')
					{
						state = 308;
						hold.setLength(0);
					}
					else 
					{
						state = 309;
						hold.append(c);
					}
					break;
				case 308:
					// <\w*[Aa]\w[Hh][Rr][Ee][Ff]\w*=\w*".*
					if (c == '>')
					{
						state = 0;
						hold.insert(0, '"');
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == '"')
					{
						// in the hold space is the URL
						Fout.write(new String("\"" + xlate.xlate(Context, hold.toString(), false) + "\"").getBytes());
						state = 301;
						hold.setLength(0);
					}
					else 
					{
						hold.append(c);
					}
					break;
				case 309:
					// <\w*[Aa]\w[Hh][Rr][Ee][Ff]\w*=\w*.*
					if (c == '>')
					{
						state = 0;
						// in the hold space is the URL
						Fout.write(new String("\"" + xlate.xlate(Context, hold.toString(), false) + "\"").getBytes());
						Fout.write(b);
						hold.setLength(0);
					}
					else if (Character.isWhitespace(c))
					{
						// in the hold space is the URL
						Fout.write(new String("\"" + xlate.xlate(Context, hold.toString(), false) + "\"").getBytes());
						Fout.write(b);
						hold.setLength(0);
						state = 301;
					}
					else 
					{
						hold.append(c);
					}
					break;
				case 400:
					// <[Ii]
					if (c == 'm' || c == 'M')
					{
						state = 401;
						hold.append(c);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 401:
					// <[Ii][Mm]
					if (c == 'g' || c == 'G')
					{
						state = 402;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 402:
					// <[Ii][Mm][Gg]\w+
					// starting on white
					if (c == '>')
					{
						state = 0;
						Fout.write(b);
					}
					else if (c == 's' || c == 'S')
					{
						state = 404;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						Fout.write(b);
					}
					else 
					{
						// in a record but lead char
						// not whitespace or [sS]
						state = 403;
						Fout.write(b);
					}
					break;
				case 403:
					// <[Ii][Mm][Gg]\w+[^Ss]+
					// gobbel in img record
					if (Character.isWhitespace(c))
					{
						state = 402;
						Fout.write(b);
					}
					else if (c == '>')
					{
						state = 0;
						Fout.write(b);
					}
					else
					{
						Fout.write(b);
					}
					break;
				case 404:
					// <[Ii][Mm][Gg]\w+[Ss]
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == 'r' || c == 'R')
					{
						state = 405;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						state = 402;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 403;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 405:
					// <[Ii][Mm][Gg]\w+[Ss][Rr]
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == 'c' || c == 'C')
					{
						state = 406;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						state = 402;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 403;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 406:
					// <[Ii][Mm][Gg]\w+[Ss][Rr][Cc]\w*
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == '=')
					{
						state = 407;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (Character.isWhitespace(c))
					{
						hold.append(c);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 407:
					// <[Ii][Mm][Gg]\w+[Ss][Rr][Cc]\w*=\w*
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (Character.isWhitespace(c))
					{
						Fout.write(b);
					}
					else if (c == '"')
					{
						state = 408;
						hold.setLength(0);
					}
					else 
					{
						state = 409;
						hold.append(c);
					}
					break;
				case 408:
					// <[Ii][Mm][Gg]\w+[Ss][Rr][Cc]\w*=\w*".*
					if (c == '>')
					{
						state = 0;
						hold.insert(0, '"');
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == '"')
					{
						// in the hold space is the URL
						Fout.write(new String("\"" + xlate.xlate(Context, hold.toString(), true) + "\"").getBytes());
						state = 402;
						hold.setLength(0);
					}
					else 
					{
						hold.append(c);
					}
					break;
				case 409:
					// <[Ii][Mm][Gg]\w+[Ss][Rr][Cc]\w*=\w*.*
					if (c == '>')
					{
						state = 0;
						// in the hold space is the URL
						Fout.write(new String("\"" + xlate.xlate(Context, hold.toString(), true) + "\"").getBytes());
						Fout.write(b);
						hold.setLength(0);
					}
					else if (Character.isWhitespace(c))
					{
						// in the hold space is the URL
						Fout.write(new String("\"" + xlate.xlate(Context, hold.toString(), true) + "\"").getBytes());
						Fout.write(b);
						hold.setLength(0);
						state = 402;
					}
					else 
					{
						hold.append(c);
					}
					break;
				case 500:
					// <[Ff]
					if (c == 'r' || c == 'R')
					{
						state = 501;
						hold.append(c);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 501:
					// <[Fr][Rr]
					if (c == 'a' || c == 'A')
					{
						state = 502;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 502:
					// <[Fr][Rr][Aa]
					if (c == 'm' || c == 'M')
					{
						state = 503;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 503:
					// <[Fr][Rr][Aa][mM]
					if (c == 'e' || c == 'E')
					{
						state = 504;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 504:
					// <[Ff][Rr][Rr][Aa][Mm][Ee]\w+
					// starting on white
					if (c == '>')
					{
						state = 0;
						Fout.write(b);
					}
					else if (c == 's' || c == 'S')
					{
						state = 506;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						Fout.write(b);
					}
					else 
					{
						// in a record but lead char
						// not whitespace or [sS]
						state = 505;
						Fout.write(b);
					}
					break;
				case 505:
					// <[Ff][Rr][Rr][Aa][Mm][Ee]\w+[^Ss]+
					// gobbel in frame record
					if (Character.isWhitespace(c))
					{
						state = 504;
						Fout.write(b);
					}
					else if (c == '>')
					{
						state = 0;
						Fout.write(b);
					}
					else
					{
						Fout.write(b);
					}
					break;
				case 506:
					// <[Ff][Rr][Rr][Aa][Mm][Ee]\w+[Ss]
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == 'r' || c == 'R')
					{
						state = 507;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						state = 504;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 505;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 507:
					// <[Ff][Rr][Rr][Aa][Mm][Ee]\w+[Ss][Rr]
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == 'c' || c == 'C')
					{
						state = 508;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						state = 504;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 505;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 508:
					// <[Ff][Rr][Rr][Aa][Mm][Ee]\w+[Ss][Rr][Cc]\w*
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == '=')
					{
						state = 509;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (Character.isWhitespace(c))
					{
						hold.append(c);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 509:
					// <[Ff][Rr][Rr][Aa][Mm][Ee]\w+[Ss][Rr][Cc]\w*=\w*
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (Character.isWhitespace(c))
					{
						Fout.write(b);
					}
					else if (c == '"')
					{
						state = 510;
						hold.setLength(0);
					}
					else 
					{
						state = 511;
						hold.append(c);
					}
					break;
				case 510:
					// <[Ff][Rr][Rr][Aa][Mm][Ee]\w+[Ss][Rr][Cc]\w*=\w*".*
					if (c == '>')
					{
						state = 0;
						hold.insert(0, '"');
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == '"')
					{
						// in the hold space is the URL
						Fout.write(new String("\"" + xlate.xlate(Context, hold.toString(), true) + "\"").getBytes());
						state = 502;
						hold.setLength(0);
					}
					else 
					{
						hold.append(c);
					}
					break;
				case 511:
					// <[Ff][Rr][Rr][Aa][Mm][Ee]\w+[Ss][Rr][Cc]\w*=\w*.*
					if (c == '>')
					{
						state = 0;
						// in the hold space is the URL
						Fout.write(new String("\"" + xlate.xlate(Context, hold.toString(), true) + "\"").getBytes());
						Fout.write(b);
						hold.setLength(0);
					}
					else if (Character.isWhitespace(c))
					{
						// in the hold space is the URL
						Fout.write(new String("\"" + xlate.xlate(Context, hold.toString(), true) + "\"").getBytes());
						Fout.write(b);
						hold.setLength(0);
						state = 502;
					}
					else 
					{
						hold.append(c);
					}
					break;
				case 600:
					// <[Bb]
					if (c == 'a' || c == 'A')
					{
						state = 601;
						hold.append(c);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 601:
					// <[Bb][Aa]
					if (c == 's' || c == 'S')
					{
						state = 602;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 602:
					// <[Bb][Aa][Ss]
					if (c == 'e' || c == 'E')
					{
						state = 603;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 603:
					// <[Bb][Aa][Ss][Ee]\w+
					// starting on white
					if (c == '>')
					{
						state = 0;
						Fout.write(b);
					}
					else if (c == 'h' || c == 'H')
					{
						state = 605;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						Fout.write(b);
					}
					else 
					{
						// in a record but lead char
						// not whitespace or [hH]
						state = 604;
						Fout.write(b);
					}
					break;
				case 604:
					// <[Bb][Aa][Ss][Ee]\w+[^Hh]+
					// gobbel in base record
					if (Character.isWhitespace(c))
					{
						state = 603;
						Fout.write(b);
					}
					else if (c == '>')
					{
						state = 0;
						Fout.write(b);
					}
					else
					{
						Fout.write(b);
					}
					break;
				case 605:
					// <[Bb][Aa][Ss][Ee]\w+[Hh]
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == 'r' || c == 'R')
					{
						state = 606;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						state = 603;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 604;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 606:
					// <[Bb][Aa][Ss][Ee]\w+[Hh][Rr]
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == 'e' || c == 'E')
					{
						state = 607;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						state = 603;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 604;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 607:
					// <[Bb][Aa][Ss][Ee]\w+[Hh][Rr][Ee]
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == 'f' || c == 'F')
					{
						state = 608;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						state = 603;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else 
					{
						state = 604;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 608:
					// <[Bb][Aa][Ss][Ee]\w+[Hh][Rr][Ee][Ff]\w*
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == '=')
					{
						state = 609;
						hold.append(c);
					}
					else if (Character.isWhitespace(c))
					{
						hold.append(c);
					}
					else 
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					break;
				case 609:
					// <[Bb][Aa][Ss][Ee]\w+[Hh][Rr][Ee][Ff]\w*=\w*
					if (c == '>')
					{
						state = 0;
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (Character.isWhitespace(c))
					{
						Fout.write(b);
					}
					else if (c == '"')
					{
						state = 610;
					}
					else 
					{
						state = 611;
						hold.append(c);
					}
					break;
				case 610:
					// <[Bb][Aa][Ss][Ee]\w+[Hh][Rr][Ee][Ff]\w*=\w*".*
					if (c == '>')
					{
						state = 0;
						hold.insert(0, '"');
						hold.append(c);
						Fout.write(hold.toString().getBytes());
						hold.setLength(0);
					}
					else if (c == '"')
					{
						// throw away the hold space HREF= .*
						state = 602;
						hold.setLength(0);
					}
					else 
					{
						hold.append(c);
					}
					break;
				case 611:
					// <[Bb][Aa][Ss][Ee]\w+[Hh][Rr][Ee][Ff]\w*=\w*.*
					if (c == '>')
					{
						state = 0;
						// throw away the hold space HREF= .*
						hold.setLength(0);
					}
					else if (Character.isWhitespace(c))
					{
						// throw away the hold space HREF= .*
						hold.setLength(0);
						state = 602;
					}
					else 
					{
						hold.append(c);
					}
					break;
			}
		}
		Fout.write(hold.toString().getBytes());
		Fout.close();
		Fin.close();
	}
}
