// Copyright (c) 2000
//      Maurice Castro (maurice@serc.rmit.edu.au)
//      All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 
// This software is provided by the original author and contributors ``as
// is'' and any express or implied warranties, including, but not limited
// to, the implied warranties of merchantability and fitness for a
// particular purpose are disclaimed.  In no event shall the authors or
// contributors be liable for any direct, indirect, incidental, special,
// exemplary, or consequential damages (including, but not limited to,
// procurement of substitute goods or services; loss of use, data, or
// profits; or business interruption) however caused and on any theory of
// liability, whether in contract, strict liability, or tort (including
// negligence or otherwise) arising in any way out of the use of this
// software, even if advised of the possibility of such damage.
//
// $Id: XlateURL.java,v 1.4 2000/11/13 00:42:18 maurice Exp $

import java.net.*;
import java.util.*;

public class XlateURL
{
	private int level;
	private Hashtable h;
	private Vector v;
	private int cnt;
	private int mode;

	public void setLevel(int level)
	{
		this.level = level;
	}
	
	public XlateURL(int mode)
	{
		h = new Hashtable();
		cnt = 0;
		this.mode = mode;
	}

	public String toString()
	{
		switch (mode)
		{
			case 0: return("Strict Tree");
			case 1: return("This host only");
			case 2: return("Any");
		}
		return("Invalid mode");
	}

	private	String suffix(String Url)
	{
		// get suffix
		String t = Url.trim().toLowerCase();
		// get filename
		int q = t.indexOf("?");
		if (q != -1)
		{
			t = t.substring(0,q);
		}
		int h = t.indexOf("#");
		if (h != -1)
		{
			t = t.substring(0,h);
		}
		int slash = t.lastIndexOf("/");
		if (slash != -1)
		{
			t = t.substring(slash+1);
		}
		// get suffix
		int dot = t.indexOf(".");
		if (dot != -1)
		{
			return(t.substring(dot));
		}
		return(".html");
	}

	public String lookup(URL url, boolean add, boolean decorate)
	{
		String urls = url.toString();
		String hashdecor = "";
		int hash = urls.indexOf("#");
                if (hash != -1)
                {
                        hashdecor = urls.substring(hash);
			urls = urls.substring(0,hash);
                }
		if (h.containsKey(urls))
		{
			if (decorate)
				return((String)h.get(urls) + hashdecor);
			else
				return((String)h.get(urls));
		}
		else
		{
			if (add)
			{
				cnt++;
				String name = cnt + suffix(urls);
				h.put(urls, name);
				if (decorate)
					return(name + hashdecor);
				else
					return(name);
			}
		}
		return urls + hashdecor;
	}

	public void setVector(Vector v)
	{
		this.v = v;
	}

	public String xlate(URL Context, String Original, boolean force) 
	{
		try
		{
			URL url = new URL(Context, Original);
			boolean inRange = false;
			switch (mode)
			{
				case 0:
					// strict tree
					String contextFile = Context.getFile();
					int slash = contextFile.lastIndexOf("/");
					if (slash != -1)
					{
						contextFile = contextFile.substring(0, slash+1);
					}
					inRange = url.getHost().equals(Context.getHost()) && 
					          url.getProtocol().equals(Context.getProtocol()) && 
					          url.getPort() == Context.getPort() && 
					          url.getFile().startsWith(contextFile);
					break;
				case 1:
					// same host
					inRange = url.getHost().equals(Context.getHost());
					break;
				case 2:
					// any host
					inRange = true;
					break;
			}
			if (inRange)
			{
				if (level > 0 || force)
				{
					v.addElement(url);
					return lookup(url, true, true);
				}
			}
			return lookup(url, false, true);
		}
		catch (Exception e) { }
		return (Original);
	}
}
